/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Contributor(s):
 *   Alexander Opitz <opitz.alexander@it-kombinat.de>
 *   Andrew Schultz <ajschult@verizon.net>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <prtypes.h>
#include <prnetdb.h>
#include <malloc.h>
#include "nsCrashRep.h"
#include "nsINIParser.h"
#include "XIErrors.h"

nsCrashRep::nsCrashRep() :
  productID(NULL), cpu(NULL), osVersion(NULL), memory(NULL), os(NULL),
  submitURL(NULL)
{
}

nsCrashRep::~nsCrashRep()
{
  if (productID)
    free(productID);
  if (cpu)
    free(cpu);
  if (osVersion)
    free(osVersion);
  if (memory)
    free(memory);
  if (os)
    free(os);
  if (submitURL)
    free(submitURL);
}

int
nsCrashRep::escapeText(const char *src, int srclen, char **dest) {
   int extrabytes = 0;
   char *specialChar = strchr(src, '&');
   while (specialChar) {
      extrabytes += 2;
      specialChar = strchr(src, '&');
   }
   *dest = (char*)malloc(srclen+extrabytes+1);
   specialChar = strchr(src, '&');
   extrabytes = 0;
   int pos = 0;
   while (specialChar) {
      strncpy(*dest+pos+extrabytes, src+pos, specialChar-src);
      pos += (specialChar-src);
      strcat(*dest+pos+extrabytes, "%26");
      pos++;
      extrabytes += 2;
      specialChar = strchr(src, '&');
   }
   strncpy(*dest+pos+extrabytes, src+pos, srclen-pos);
   (*dest)[srclen+extrabytes] = '\0';
   return OK;
}

int
nsCrashRep::fixupParameter(char **parameter) {
   static char buf[10240];
   if (**parameter == '<') {
      int fd = open((*parameter)+1, O_RDONLY);
      if (!fd)
        return E_NO_FILE;
      ssize_t nbytes = read(fd, buf, 10239);
      close(fd);
      buf[10239] = '\0';
      int extrabytes = 0;
      char *specialChar = strchr(buf, '&');
      while (specialChar) {
         extrabytes += 2;
         specialChar = strchr(buf, '&');
      }
      free(*parameter);
      escapeText(buf,nbytes,parameter);
   }
   return OK;
}

int
nsCrashRep::ParseGeneral(nsINIParser *aParser)
{
  int     err = OK;
  char    *buf = NULL;
  int     size = 0;

  err = aParser->GetStringAlloc("General", "productid", &buf, &size);
  if (err == OK && size > 0)
    productID = buf;
  else
    err = E_PARAM;

  if (err == OK) {
    err = aParser->GetStringAlloc("General", "os", &buf, &size);
    if (err == OK && size > 0)
      os = buf;
    else
      err = E_PARAM;
  }

  if (err == OK) {
    err = aParser->GetStringAlloc("General", "cpu", &buf, &size);
    if (err == OK && size > 0) {
      fixupParameter(&buf);
      cpu = buf;
    }
    else {
      err = E_PARAM;
    }
  }
  
  if (err == OK) {
    err = aParser->GetStringAlloc("General", "osversion", &buf, &size);
    if (err == OK && size > 0) {
      fixupParameter(&buf);
      osVersion = buf;
    }
    else {
      err = E_PARAM;
    }
  }
  
  if (err == OK) {
    err = aParser->GetStringAlloc("General", "memory", &buf, &size);
    if (err == OK && size > 0) {
      memory = buf;
      fixupParameter(&memory);
    }
    else {
      err = E_PARAM;
    }
  }
  
  if (err == OK) {
    err = aParser->GetStringAlloc("General", "SubmitURL", &buf, &size);
    if (err == OK && size > 0)
      submitURL = buf;
    else
      err = E_PARAM;
  }

  return err;
}

int
nsCrashRep::ParseArgs(int aArgc, char **aArgv) {
  if (aArgc < 4) {
    fprintf(stderr, "usage: crashrep trace map signal\n");
    return 1;
  }

  trace = aArgv[1];
  maps = aArgv[2];
  signal = aArgv[3];

  return OK;
}
