/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Contributor(s):
 *   Alexander Opitz <opitz.alexander@it-kombinat.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <prtypes.h>
#include <prnetdb.h>
#include <malloc.h>
#include <gtk/gtk.h>
#include "nsNetStub.h"
#include "nsINIParser.h"
#include "XIErrors.h"
#include "logo.xpm"
#include "nsCrashRepGTK.h"

#define XI_GTK_UPDATE_UI()                              \
do {                                                    \
    while (gtk_events_pending())                        \
        gtk_main_iteration();                           \
} while (0);

nsCrashRepGTK::nsCrashRepGTK() :
  progressBar(NULL), emailBox(NULL), URLBox(NULL), commentBox(NULL)
{
}

/*
 * When invoked (via signal delete_event), terminates the application.
 */
gint CloseApplication(GtkWidget *widget, GdkEvent *event, gpointer data) {
    gtk_main_quit();
    return(FALSE);
}

void CloseButtonClicked(GtkWidget *widget, gpointer data) {
   GtkWidget *dlg = (GtkWidget *) data;
   if (dlg)
      gtk_widget_destroy(dlg);
}

void ExamineButtonClicked(GtkWidget *widget, gpointer data) {
  nsCrashRepGTK *crashRep = (nsCrashRepGTK*)data;
  
  GtkWidget *dataDialog = gtk_dialog_new();
  gtk_window_set_modal(GTK_WINDOW(dataDialog), TRUE);
  gtk_window_set_title(GTK_WINDOW(dataDialog), "Crash Data");
  gtk_window_set_position(GTK_WINDOW(dataDialog), GTK_WIN_POS_CENTER);
  gtk_container_set_border_width(GTK_CONTAINER(dataDialog), 10);

#ifdef MOZ_WIDGET_GTK2
  GtkWidget *dataText = gtk_text_view_new();
  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(dataText), GTK_WRAP_WORD);
  GtkWidget *textWindow = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (textWindow),
                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(textWindow), dataText);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(dataText), FALSE);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dataDialog)->vbox), textWindow);
  GtkTextBuffer *dataBuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(dataText));
  GtkTextIter iter;
#define INSERT_TEXT(string) { \
   gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(dataBuf), &iter); \
   gtk_text_buffer_insert(GTK_TEXT_BUFFER(dataBuf), &iter, (string), -1);\
}
#else
  GtkWidget *dataText = gtk_text_new(NULL,NULL);
  gtk_text_set_word_wrap(GTK_TEXT(dataText), TRUE);
  gtk_text_set_line_wrap(GTK_TEXT(dataText), TRUE);
  gtk_editable_set_editable(GTK_EDITABLE(dataText), FALSE);
  GtkWidget *vscrollbar = gtk_vscrollbar_new (GTK_TEXT(dataText)->vadj);
  GtkWidget *table = gtk_table_new(1, 2, FALSE);
  gtk_table_attach(GTK_TABLE(table), dataText, 0, 1, 0, 1,
                   static_cast<GtkAttachOptions>(GTK_EXPAND|GTK_FILL),
                   static_cast<GtkAttachOptions>(GTK_EXPAND|GTK_FILL), 0, 0 );
  gtk_table_attach(GTK_TABLE(table), vscrollbar, 1, 2, 0, 1,
                   GTK_FILL,
                   static_cast<GtkAttachOptions>(GTK_SHRINK|GTK_EXPAND|GTK_FILL), 0, 0 );
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dataDialog)->vbox), table);
#define INSERT_TEXT(string) { \
   gtk_text_insert(GTK_TEXT(dataText), NULL, NULL, NULL, (string), -1);\
}
#endif

  gtk_widget_set_usize(dataText, 500, 450);
  
  INSERT_TEXT("ProductID: ")
  INSERT_TEXT(crashRep->productID)
  INSERT_TEXT("\nOS: ")
  INSERT_TEXT(crashRep->os)
  INSERT_TEXT("\nCPU\n")
  INSERT_TEXT(crashRep->cpu)
  INSERT_TEXT("\nOS version: ")
  INSERT_TEXT(crashRep->osVersion)
  INSERT_TEXT("\nMemory\n")
  INSERT_TEXT(crashRep->memory)
  INSERT_TEXT("\nBacktrace\n")
  INSERT_TEXT(crashRep->trace)
  INSERT_TEXT("\n\nMemmory Mapping\n")
  INSERT_TEXT(crashRep->maps)
  INSERT_TEXT("\nSignal\n")
  INSERT_TEXT(crashRep->signal)

  GtkWidget *closeButton = gtk_button_new_with_label("Close window");
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dataDialog)->action_area), closeButton);

  gtk_signal_connect(GTK_OBJECT(closeButton), "clicked",
                     GTK_SIGNAL_FUNC(CloseButtonClicked), dataDialog );

  gtk_widget_show_all(dataDialog);
}

/* is invoked when the button is clicked.  It just prints a message.
*/

void SendButtonClicked(GtkWidget *widget, gpointer data) {
  nsCrashRepGTK *crashRep = (nsCrashRepGTK*)data;
  
    PRHostEnt hostentry;
    char *buf;
    PRNetAddr addr;
    PRFileDesc *socket = NULL;
    char err[250];
    PRIntn cmdSize;
    char host[HOST_SIZE];
    char port[PORT_SIZE];
    char path[PATH_SIZE];
    char line[LINE_SIZE];
    PRBool endOfHeader = PR_FALSE;
    PRBool endOfContent = PR_FALSE;
    PRUint32 fileSize;
    gtk_widget_set_sensitive(widget, 0);
    XI_GTK_UPDATE_UI();
    if (nsNetStub::ParseURL(crashRep->submitURL, host, sizeof(host), port,
                            sizeof(port), path, sizeof(path)) == PR_FAILURE) {
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar),
                                       "Configuration error");
	return;
    }
    gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar),
                                   "Looking up host");
    XI_GTK_UPDATE_UI();
    char prbuf[PR_NETDB_BUF_SIZE];
    if (PR_GetHostByName(host, prbuf, sizeof(prbuf), &hostentry)
	    == PR_FAILURE) {
        sprintf(err, "Unknown host name: %s\n", host);
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
	return;
    }
    gtk_progress_set_value(GTK_PROGRESS (crashRep->progressBar), 5);
    XI_GTK_UPDATE_UI();

    addr.inet.family = PR_AF_INET;
    addr.inet.port = PR_htons((short) atoi(port));
    addr.inet.ip = *((PRUint32 *) hostentry.h_addr_list[0]);

    gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), "Contacting host");
    XI_GTK_UPDATE_UI();
    socket = PR_NewTCPSocket();
    if (socket == NULL) {
	sprintf(err, "cannot create new tcp socket\n");
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
	return;
    }
    gtk_progress_set_value(GTK_PROGRESS (crashRep->progressBar), 7);
    XI_GTK_UPDATE_UI();
    
    if (PR_Connect(socket, &addr, PR_INTERVAL_NO_TIMEOUT) == PR_FAILURE) {
	sprintf(err, "Cannot connect to server\n");
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
	return;
    }
    gtk_progress_set_value(GTK_PROGRESS (crashRep->progressBar), 12);
    gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), "Send data to server");
    XI_GTK_UPDATE_UI();

    char *emailStr, *URLStr, *commentStr;
    const char *gbuf = gtk_entry_get_text(GTK_ENTRY(crashRep->emailBox));
    nsCrashRep::escapeText(gbuf, strlen(gbuf), &emailStr);
    gbuf = gtk_entry_get_text(GTK_ENTRY(crashRep->URLBox));
    nsCrashRep::escapeText(gbuf, strlen(gbuf), &URLStr);
#ifdef MOZ_WIDGET_GTK2
    GtkTextBuffer *textBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(crashRep->commentBox));
    GtkTextIter start, end;
    gtk_text_buffer_get_bounds(textBuffer, &start, &end);
    char *commentBoxStr = gtk_text_buffer_get_text(textBuffer,&start,&end,FALSE);
#else
    char *commentBoxStr = gtk_editable_get_chars(GTK_EDITABLE(commentBox),0,-1);
#endif
    nsCrashRep::escapeText(commentBoxStr, strlen(commentBoxStr), &commentStr);
    g_free(commentBoxStr);

    unsigned int toPostLength = strlen("trace=");
    toPostLength+=strlen("&maps=");
    toPostLength+=strlen("&signal=");
    toPostLength+=strlen("&productid=");
    toPostLength+=strlen("&os=");
    toPostLength+=strlen("&cpu=");
    toPostLength+=strlen("&osversion=");
    toPostLength+=strlen("&memory=");
    toPostLength+=strlen("&email=");
    toPostLength+=strlen("&url=");
    toPostLength+=strlen("&comment=");
    toPostLength+=strlen(crashRep->trace);
    toPostLength+=strlen(crashRep->maps);
    toPostLength+=strlen(crashRep->signal);
    toPostLength+=strlen(crashRep->productID);
    toPostLength+=strlen(crashRep->os);
    toPostLength+=strlen(crashRep->cpu);
    toPostLength+=strlen(crashRep->osVersion);
    toPostLength+=strlen(crashRep->memory);
    toPostLength+=strlen(emailStr);
    toPostLength+=strlen(URLStr);
    toPostLength+=strlen(commentStr);
    
    char *toPost = (char*)malloc(toPostLength+1);
    sprintf(toPost, "trace=%s&maps=%s&signal=%s&productid=%s&os=%s&cpu=%s"
                    "&osversion=%s&memory=%s&email=%s&url=%s&comment=%s", 
                    crashRep->trace, crashRep->maps, crashRep->signal,
                    crashRep->productID, crashRep->os, crashRep->cpu,
                    crashRep->osVersion, crashRep->memory, emailStr, URLStr,
                    commentStr);

    if (toPostLength != strlen(toPost)) {
       fprintf(stderr, "you screwed up %d %d\n", toPostLength, strlen(toPost));
       fprintf(stderr, "|%s|\n", toPost);
       return;
    }
    
    int bufSize = toPostLength+1+PR_NETDB_BUF_SIZE;
    buf = (char*)malloc(bufSize);
    cmdSize = snprintf(buf, bufSize, "POST %s HTTP/1.1\r\nhost:%s\r\n"
                                          "User-Agent: SeaMonkey Crash Reporter 0.1\r\n"
                                          "Accept: text/plain\r\n"
                                          "Accept-Charset: iso-8859-1\r\n"
                                          "Cache-Control: max-age=0\r\n"
                                          "Content-Type: application/x-www-form-urlencoded\r\n"
                                          "Content-Length: %i\r\n"
                                          "\r\n"
                                          "%s", path,host, toPostLength, toPost);
    
    if (PR_Write(socket, buf, cmdSize) != cmdSize) {
	sprintf(err, "Error sending data to server\n");
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
	return;
    }

    gtk_progress_set_value(GTK_PROGRESS (crashRep->progressBar), 20);
    gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar),
                                   "Waiting for response...");
    XI_GTK_UPDATE_UI();

    nsNetStub netStub;
    if (netStub.ReadLine(socket, line, sizeof(line)) <= 0) {
	sprintf(err, "Error parsing response from server\n");
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
	return;
    }

    gtk_progress_set_value(GTK_PROGRESS (crashRep->progressBar), 40);
    XI_GTK_UPDATE_UI();

    /* HTTP response: 200 == OK */
    if (strstr(line, "200") == NULL) {
	sprintf(err, "%s\n", line);
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
	return;
    }

    gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), "Reading from Server...");
    XI_GTK_UPDATE_UI();

    fileSize=0;
    while (netStub.ReadLine(socket, line, sizeof(line)) > 0) {
	if (line[0] == '\n') {
	    endOfHeader = PR_TRUE;
	    break;
	}
	if (strncmp(line, "Content-Length", 14) == 0
		|| strncmp(line, "Content-length", 14) == 0) {
	    char *p = line + 14;

	    while (*p == ' ' || *p == '\t') {
		p++;
	    }
	    if (*p != ':') {
		continue;
            }
	    p++;
	    while (*p == ' ' || *p == '\t') {
		p++;
	    }
	    fileSize = 0;
	    while ('0' <= *p && *p <= '9') {
		fileSize = 10 * fileSize + (*p - '0');
		p++;
            }
	}
    }

    if (endOfHeader == PR_FALSE) {
        sprintf(err, "Error parsing response from server\n");
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
        return;
    }
    if (netStub.ReadLine(socket, line, sizeof(line)) <= 0 ||
        netStub.ReadLine(socket, line, sizeof(line)) <= 0) {
        sprintf(err, "Error parsing response from server\n");
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
        return;
    }

    int n = strlen(line);
    line[n-1] = '\0';
    sprintf(err, "Your Crash Incident ID is %s", line);
    gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
    XI_GTK_UPDATE_UI();

    while (netStub.ReadLine(socket, line, sizeof(line)) > 0) {
        if (line[0] == '\n') {
            endOfContent = PR_TRUE;
            break;
        }
    }

    if (endOfContent == PR_FALSE) {
        sprintf(err, "cannot read content from http server\n");
        gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), err);
        return;
    }
    gtk_progress_set_value(GTK_PROGRESS (crashRep->progressBar), 60);
    XI_GTK_UPDATE_UI();

    if (socket) PR_Close(socket);
    gtk_progress_set_value(GTK_PROGRESS (crashRep->progressBar), 100);
    XI_GTK_UPDATE_UI();
}

int main(int argc, char *argv[])
{
    GtkWidget *window, *pixmapwid, *quitbutton, *vbox, *frame, *frame2, *label;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    GtkStyle *style;
    GtkWidget *table;

    int pathLength;
    char programdir[256];
    char inifile[256];
    int err = OK;

    nsCrashRepGTK *crashRep = new nsCrashRepGTK();
    if (!crashRep) {
      return E_MEM;
    }
    if ((err = crashRep->ParseArgs(argc, argv)) != OK)  {
      return err;
    }

    if ((pathLength = readlink("/proc/self/exe", programdir, 255)) < 1) {
      fprintf(stderr, "Could not locate program directory\n");
      return 1;
    }
    programdir[pathLength] = '\0';
    char* filename = rindex(programdir, '/');
    *filename = '\0';
    snprintf(inifile, 255, "%s/%s", programdir, "crashrep.ini");
    inifile[255] = '\0';

    nsINIParser *parser = new nsINIParser(inifile);

    if (!parser) {
      // out of memory
      return 1;
    }

    err = parser->GetError();
    if (err != nsINIParser::OK)
        return err;

    err = crashRep->ParseGeneral(parser);
    if (err != nsINIParser::OK)
        return err;

    /* create the main window, and attach delete_event signal to terminating
       the application */
    gtk_init( &argc, &argv );
    window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
    gtk_window_set_title( GTK_WINDOW( window ), "Crash Reporting Tool");
    gtk_signal_connect( GTK_OBJECT (window), "delete_event",
                        GTK_SIGNAL_FUNC (CloseApplication), NULL );
    gtk_container_set_border_width( GTK_CONTAINER (window), 10 );
    gtk_widget_show( window );

    /* now for the pixmap from gdk */
    style = gtk_widget_get_style( window );

    pixmap = gdk_pixmap_create_from_xpm_d( window->window,  &mask,
                                           &style->bg[GTK_STATE_NORMAL],
                                           logo_xpm );
    /* a pixmap widget to contain the pixmap */
    pixmapwid = gtk_pixmap_new( pixmap, mask );
    gtk_widget_show( pixmapwid );

    vbox = gtk_vbox_new(FALSE, 5);
    table = gtk_table_new(4, 2, FALSE);

    GtkWidget *sendButton = gtk_button_new_with_label("Send crash data");
    GtkWidget *examineButton = gtk_button_new_with_label("Examine crash data");
    GtkWidget *buttonBox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(buttonBox), sendButton, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(buttonBox), examineButton, TRUE, TRUE, 0);

    crashRep->progressBar = gtk_progress_bar_new();
    gtk_progress_set_value(GTK_PROGRESS (crashRep->progressBar), 0);
    gtk_progress_set_format_string(GTK_PROGRESS (crashRep->progressBar), "");
    gtk_progress_set_show_text(GTK_PROGRESS (crashRep->progressBar), TRUE);
    quitbutton = gtk_button_new_with_label("Exit Crash Reporting Tool");
    
    frame = gtk_frame_new("SORRY!");
    frame2 = gtk_frame_new("What will be sent out");
    label = gtk_label_new("We are so sorry about crashing, but please help us\n"
                          "and send us data about your crash reporting so we\n"
                          "can fix the problem in the future.");
    gtk_container_add(GTK_CONTAINER(frame), label);

    GtkWidget *entryFrame = gtk_frame_new("Additional Information (optional)");
    GtkWidget *entryTable = gtk_table_new(4, 2, FALSE);
    gtk_container_set_border_width( GTK_CONTAINER (entryTable), 10 );

    crashRep->emailBox = gtk_entry_new();
    gtk_editable_set_editable(GTK_EDITABLE(crashRep->emailBox), TRUE);
    gtk_widget_show(crashRep->emailBox);
    GtkWidget *boxLabel = gtk_label_new("Email");
    gtk_widget_show(boxLabel);
    gtk_misc_set_alignment(GTK_MISC(boxLabel), 0, 0);
    gtk_table_attach(GTK_TABLE(entryTable), boxLabel, 0, 1, 0, 1,
                     static_cast<GtkAttachOptions>(GTK_FILL),
                     static_cast<GtkAttachOptions>(GTK_EXPAND|GTK_FILL), 7, 0 );
    gtk_table_attach_defaults( GTK_TABLE(entryTable), crashRep->emailBox, 1, 2, 0, 1 );

    crashRep->URLBox = gtk_entry_new();
    gtk_editable_set_editable(GTK_EDITABLE(crashRep->URLBox), TRUE);
    gtk_widget_show(crashRep->URLBox);
    boxLabel = gtk_label_new("URL");
    gtk_misc_set_alignment(GTK_MISC(boxLabel), 0, 0);
    gtk_label_set_justify(GTK_LABEL(boxLabel), GTK_JUSTIFY_LEFT);
    gtk_widget_show(boxLabel);
    gtk_table_attach(GTK_TABLE(entryTable), boxLabel, 0, 1, 1, 2,
                     static_cast<GtkAttachOptions>(GTK_FILL),
                     static_cast<GtkAttachOptions>(GTK_EXPAND|GTK_FILL), 7, 0 );
    gtk_table_attach_defaults( GTK_TABLE(entryTable), crashRep->URLBox, 1, 2, 1, 2);

#ifdef MOZ_WIDGET_GTK2
    crashRep->commentBox = gtk_text_view_new();
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(crashRep->commentBox), GTK_WRAP_WORD);
    GtkWidget *commentWindow = gtk_scrolled_window_new(NULL,NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (commentWindow),
                      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(commentWindow), crashRep->commentBox);
#else
    crashRep->commentBox = gtk_text_new(NULL,NULL);
    gtk_text_set_word_wrap(GTK_TEXT(crashRep->commentBox), TRUE);
    gtk_text_set_line_wrap(GTK_TEXT(crashRep->commentBox), TRUE);
    gtk_editable_set_editable(GTK_EDITABLE(crashRep->commentBox), TRUE);
    GtkWidget *vscrollbar = gtk_vscrollbar_new (GTK_TEXT(crashRep->commentBox)->vadj);
    GtkWidget *commentTable = gtk_table_new(1, 2, FALSE);
    gtk_table_attach(GTK_TABLE(commentTable), crashRep->commentBox, 0, 1, 0, 1,
                     static_cast<GtkAttachOptions>(GTK_EXPAND|GTK_FILL),
                     static_cast<GtkAttachOptions>(GTK_EXPAND|GTK_FILL), 0, 0 );
    gtk_table_attach(GTK_TABLE(commentTable), vscrollbar, 1, 2, 0, 1,
                     GTK_FILL,
                     static_cast<GtkAttachOptions>(GTK_SHRINK|GTK_EXPAND|GTK_FILL), 0, 0 );
#endif
    gtk_widget_set_usize(crashRep->commentBox, 400, 150);
    gtk_widget_show(crashRep->commentBox);
    boxLabel = gtk_label_new("Comment");
    gtk_misc_set_alignment(GTK_MISC(boxLabel), 0, 0);
    gtk_label_set_justify(GTK_LABEL(boxLabel), GTK_JUSTIFY_LEFT);
    gtk_widget_show(boxLabel);
    gtk_table_attach_defaults( GTK_TABLE(entryTable), boxLabel, 0, 2, 2, 3);
#ifdef MOZ_WIDGET_GTK2
    gtk_table_attach_defaults( GTK_TABLE(entryTable), commentWindow, 0, 2, 3, 4);
#else
    gtk_table_attach_defaults( GTK_TABLE(entryTable), commentTable, 0, 2, 3, 4);
#endif
    gtk_container_add(GTK_CONTAINER(entryFrame), entryTable);
    gtk_box_pack_start(GTK_BOX(vbox), entryFrame, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(vbox), buttonBox, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), crashRep->progressBar, FALSE, FALSE, 0);
    gtk_container_set_border_width( GTK_CONTAINER (frame), 10 );
    gtk_container_set_border_width( GTK_CONTAINER (vbox), 10 );

    gtk_table_attach_defaults( GTK_TABLE( table ), pixmapwid, 0, 1, 0, 1 );

    gtk_table_attach_defaults( GTK_TABLE( table ), vbox, 1, 2, 0, 1 );
    gtk_table_attach_defaults( GTK_TABLE( table ), quitbutton, 0, 2, 1, 2 );

    vbox = gtk_vbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
    gtk_container_add( GTK_CONTAINER( window ), vbox );

    gtk_signal_connect( GTK_OBJECT(sendButton), "clicked",
                        GTK_SIGNAL_FUNC(SendButtonClicked), crashRep);

    gtk_signal_connect( GTK_OBJECT(examineButton), "clicked",
                        GTK_SIGNAL_FUNC(ExamineButtonClicked), crashRep);

    gtk_signal_connect_object( GTK_OBJECT( quitbutton ), "clicked",
                        GTK_SIGNAL_FUNC( gtk_main_quit ),
                        GTK_OBJECT( window ) );

    /* show the window */
    gtk_widget_show_all(window);
    gtk_main ();
          
    return 0;
}
/* example-end */

